classdef outputWithPedigree
% Class used to represent variables stored in files.
%
% Copyright (C) 2013-2014  Joao Hespanha    

    properties
        pedigreeName
        fileName
        variableName
        saveformat='serial';
                           % 'plainsave' seems to be the same as 'v7
                           % 'v7'   much faster than -v7.3
                           % 'v7.3' slower, but seems needed for large files
                           % 'serial' very fast but seems to cause a crash for large data
    end
    
    methods
        function obj=outputWithPedigree(pedigreeName,fileName,variableName)
            obj.pedigreeName=pedigreeName;
            obj.fileName=fileName;
            obj.variableName=variableName;
            switch obj.saveformat
              case 'plainsave' % probably same as v7
              case 'v7'
              case 'v7.3'
              case 'serial'
                obj.fileName=regexprep(obj.fileName,'.mat',''); 
              otherwise
                error('%s: unkown save format ''%s''\n',caller,obj.saveformat);
            end
        end
        
        function saveValue(obj,value,caller)
            if nargin>=3
                fprintf('%s: saving %s -> %s... ',caller,obj.variableName,obj.fileName);
                t0=clock;
            end
            switch obj.saveformat
              case 'plainsave' % probably same as v7
                assign(obj.variableName,value);
                save(obj.fileName,obj.variableName); 
              case 'v7'
                assign(obj.variableName,value);
                save(obj.fileName,obj.variableName,'-v7');  % much faster than -v7.3
              case 'v7.3'
                assign(obj.variableName,value);
                save(obj.fileName,obj.variableName,'-v7.3'); % very slow but needed for large files
              case 'serial'
                serialsave(obj.fileName,value);
              otherwise
                error('%s: unkown save format ''%s''\n',caller,obj.saveformat);
            end
            if nargin>=3
                fprintf('done (%.3f sec)\n',etime(clock,t0));
            end
        end
        
        function value=getValue(obj,caller)
            if nargin>=2
                fprintf('%s: loading %s <- %s... ',caller,obj.variableName,obj.fileName);
                t0=clock;
            end
            switch obj.saveformat
              case {'plainsave','v7','v7.3'}
                load(obj.fileName,obj.variableName);
                value=eval(obj.variableName);
              case 'serial'
                value=serialload(obj.fileName);
              otherwise
                error('%s: unkown save format ''%s''\n',caller,obj.saveformat);
            end
            if nargin>=2
                fprintf('done (%.3f sec)\n',etime(clock,t0));
            end
        end
        
        function disp(obj)
            fprintf('pedigreeName = %s\n',obj.pedigreeName);
            fprintf('fileName     = %s\n',obj.fileName);
            fprintf('variableName = %s\n',obj.variableName);
        end
    
    end
    
end
